<?php
/* --------------------------------------------------------------
   AutoUpdaterController.inc.php 2018-02-01
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('AdminHttpViewController');

/**
 * Class AutoUpdaterController
 */
class AutoUpdaterController extends AdminHttpViewController
{
	/**
	 * @var \UpdateDownloader
	 */
	private $updateDownloader;
	
	/**
	 * @var \messageStack_ORIGIN
	 */
	private $messageStack;
	
	/**
	 * @var \LanguageTextManager
	 */
	private $textPhrases;
	
	
	/**
	 * Initial method for this controller.
	 */
	public function init()
	{
		$this->updateDownloader = MainFactory::create('UpdateDownloader');
		$this->messageStack     = $GLOBALS['messageStack'];
		$this->textPhrases      = MainFactory::create('LanguageTextManager', 'update_downloader',
		                                              $_SESSION['languages_id']);
	}
	
	
	/**
	 * Default action.
	 * Shows the current and available version and also all available updates.
	 *
	 * @return \AdminLayoutHttpControllerResponse
	 */
	public function actionDefault()
	{
		$title    = new NonEmptyStringType($this->textPhrases->get_text('PAGE_TITLE'));
		$template = new ExistingFile(new NonEmptyStringType(DIR_FS_ADMIN
		                                                    . '/html/content/update_downloader/overview.html'));
		
		$data = MainFactory::create('KeyValueCollection', [
			'updateStatus'         => $this->updateDownloader->getUpdatesStatus(),
			'downloadedUpdates'    => $this->updateDownloader->getDownloadedUpdates(),
			'downloadBeforeUpdate' => $this->updateDownloader->getDownloadBeforeUpdate(),
			'uninstalledUpdates'   => $this->checkForUninstalledUpdates(),
		]);
		
		$assets = MainFactory::create('AssetCollection');
		$assets->add(MainFactory::create('Asset',
		                                 DIR_WS_CATALOG . 'admin/html/assets/styles/legacy/update_downloader.css'));
		$assets->add(MainFactory::create('Asset', DIR_WS_CATALOG . 'JSEngine/build/libs/loading_spinner.min.js'));
		$assets->add(MainFactory::create('Asset', DIR_WS_CATALOG
		                                          . 'admin/html/assets/javascript/engine/compatibility/update_downloader.js'));
		$assets->add(MainFactory::create('Asset', 'update_downloader.lang.inc.php'));
		
		return MainFactory::create('AdminLayoutHttpControllerResponse', $title, $template, $data, $assets);
	}
	
	
	/**
	 * Download updates action.
	 * Downloads all available updates and redirects to the default page.
	 *
	 * @return \RedirectHttpControllerResponse
	 */
	public function actionDownloadUpdates()
	{
		try
		{
			$this->updateDownloader->downloadUpdates();
		}
		catch(Exception $e)
		{
			if($e instanceof UpdateDownloaderServerNotAvailableException)
			{
				$this->messageStack->add_session($e->getMessage(), 'error');
			}
			elseif($e instanceof UpdateDownloaderNoUpdateAvailableException)
			{
				$this->messageStack->add_session($e->getMessage(), 'info');
			}
			elseif($e instanceof UpdateDownloaderDownloadFailedException === false)
			{
				throw $e;
			}
		}
		
		return MainFactory::create('RedirectHttpControllerResponse', 'admin.php?do=AutoUpdater');
	}
	
	
	/**
	 * Install updates action.
	 * May download all available updates and unzips them. Checks the file permissions and copy the files into the shop.
	 *
	 * @return \RedirectHttpControllerResponse
	 */
	public function actionInstallUpdates()
	{
		try
		{
			$this->updateDownloader->installUpdates();
		}
		catch(Exception $e)
		{
			if($e instanceof UpdateDownloaderServerNotAvailableException
			   || $e instanceof UpdateDownloaderNoUpdateAvailableException
			   || $e instanceof UpdateDownloaderBackupRestoreFailedException
			   || $e instanceof UpdateDownloaderUpdateFailedException)
			{
				$this->messageStack->add_session($e->getMessage(), 'error');
			}
			elseif($e instanceof UpdateDownloaderNoUpdateAvailableException)
			{
				$this->messageStack->add_session($e->getMessage(), 'info');
			}
			elseif($e instanceof UpdateDownloaderDownloadFailedException === false
			       && $e instanceof UpdateDownloaderBackupCreationFailedException === false)
			{
				throw $e;
			}
			
			return MainFactory::create('RedirectHttpControllerResponse', 'admin.php?do=AutoUpdater');
		}
		
		return MainFactory::create('RedirectHttpControllerResponse', '../gambio_updater');
	}
	
	
	/**
	 * Checks the version info directory and version history database table for uninstalled updates.
	 *
	 * @return bool Returns true, if there is an uninstalled update, otherwise false.
	 */
	public function checkForUninstalledUpdates()
	{
		foreach(new DirectoryIterator(DIR_FS_CATALOG . 'version_info') as $directoryItem)
		{
			if($directoryItem->isDot() || $directoryItem->isDir()
			   || $directoryItem->getExtension() !== 'php'
			   || strpos($directoryItem->getFilename(), 'google_services') === 0
			   || strpos($directoryItem->getFilename(), 'gambio_hub') === 0
			   || strpos($directoryItem->getFilename(), 'update_downloader') === 0
			   || strpos($directoryItem->getFilename(), 'ovisto') === 0)
			{
				continue;
			}
			
			$version = $this->determineVersion(new NonEmptyStringType($directoryItem->getPathname()));
			if(!empty($version) && !$this->checkVersionIsInstalled($version))
			{
				return true;
			}
		}
		
		return false;
	}
	
	
	/**
	 * Determines the version of the version info items while considering known edge cases.
	 *
	 * @param \NonEmptyStringType $versionInfoFilePath
	 *
	 * @return string $version
	 */
	protected function determineVersion(NonEmptyStringType $versionInfoFilePath)
	{
		$versionInfoFilePath = $versionInfoFilePath->asString();
		$version             = ' ';
		
		$directoryItemFile = fopen($versionInfoFilePath, 'r');
		while($line = fgets($directoryItemFile))
		{
			if(strpos($line, 'version:') === 0)
			{
				$version = trim(str_replace('version:', '', $line));
				
				break;
			}
		}
		fclose($directoryItemFile);
		
		return trim($version);
	}
	
	
	/**
	 * Checks the version history database table if the given version is installed.
	 *
	 * @return bool Returns true, if the given version is installed, otherwise false.
	 */
	public function checkVersionIsInstalled($version)
	{
		$db      = StaticGXCoreLoader::getDatabaseQueryBuilder();
		$version = $db->select()->from('version_history')->where(['name' => $version])->or_where([
			                                                                                         'name' => 'v'
			                                                                                                   . $version
		                                                                                         ])->get()->row_array();
		
		if(isset($version['version']))
		{
			return true;
		}
		
		return false;
	}
}